<?php
/**
 * Initializes the database layer
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// Initialize layer
$layer = null;

// Is the type defined?
if (!defined('_DB_TYPE')) {
	// Abort here
	ApplicationEntryPoint::app_die("[Main:] Please define a database type first!<br />
E.g.:<pre>&#36;cfg->defineDatabaseType(&quot;local&quot;);</pre>
This will choose the local file-based database layer.");
}

// Generate FQFN for the database layer
$INC = sprintf("%sinc/database/lib-%s%s",
	PATH,
	_DB_TYPE,
	FrameworkConfiguration::getInstance()->readConfig('php_extension')
);

// Load the database layer include
if ((file_exists($INC)) && (is_file($INC)) && (is_readable($INC))) {
	// Load the layer
	require_once($INC);
} else {
	// Layer is missing!
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database layer is missing! (%s) -&gt; R.I.P.",
		_DB_TYPE
	));
}

// Clean it up
unset($INC);

// Prepare database instance
try {
	$db = DatabaseConnection::createDatabaseConnection(DebugMiddleware::getInstance(), $layer);
} catch (NullPointerException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
} catch (InvalidDirectoryResourceException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
} catch (PathIsEmptyException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
} catch (PathIsNoDirectoryException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
} catch (PathReadProtectedException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
} catch (DirPointerNotOpenedException $e) {
	ApplicationEntryPoint::app_die(sprintf("[Main:] Database sub-system not initialized for the follwing reason: <strong>%s</strong><br />\n",
		$e->getMessage()
	));
}

// Datenbankobjekt debuggen
if (defined('DEBUG_DATABASE_OBJ')) {
	DebugMiddleware::getInstance()->output(sprintf("The database sub-system does now look like this:<br />
<pre>%s</pre>\n",
		print_r($db, true)
	));
}

// [EOF]
?>
